#include "machineReassignment.h"

void MachineReassignment::ConflictConstraint::reset(){
  nonzero.clear();
  zero.clear();
  for (unsigned int m = 0; m < con->mr->numMachines; ++m){
    counts[m] = 0;
    index[m] = m;
    zero.push_back(m);
  }
}

void MachineReassignment::ConflictConstraint::increaseCount(unsigned int machine){
  if (++counts[machine] == 1){
    unsigned int tail = *zero.rbegin();
    zero[index[machine]] = tail;
    index[tail] = index[machine];
    zero.pop_back();
    index[machine] = nonzero.size();
    nonzero.push_back(machine);
  }
  if (counts[machine] > 1) ++penalty;
}

void MachineReassignment::ConflictConstraint::decreaseCount(unsigned int machine){
  if (--counts[machine] == 0){
    unsigned int tail = *nonzero.rbegin();
    nonzero[index[machine]] = tail;
    index[tail] = index[machine];
    nonzero.pop_back();
    index[machine] = zero.size();
    zero.push_back(machine);
  }
  if (counts[machine] > 0) --penalty;
}

MachineReassignment::int64 MachineReassignment::ConflictConstraint::calculatePenalty(Solution &solution){
  penalty = 0;
  reset();
  for (vector<unsigned int>::const_iterator ite = processes->begin(); ite != processes->end(); ++ite){
    increaseCount(solution.assignment[*ite]);
  }
  return penalty;
}

MachineReassignment::int64 MachineReassignment::ConflictConstraint::updatePenalty(Solution &solution, Solver::Move *move){
  ppenalty = penalty;
  switch(move->type){
  case Solver::Shift:
    {
      Solver::ShiftMove *shift = static_cast<Solver::ShiftMove*>(move);
      if (con->solver->processes[shift->process].service == service){
        decreaseCount(solution.assignment[shift->process]);
        increaseCount(shift->machine);
      }
      break;
    }
  case Solver::Swap:
    {
      // Do Nothing
      break;
    }
  }
  return penalty - ppenalty;
}

void MachineReassignment::ConflictConstraint::calculateDiff(Solution &solution){
  // calculate diff; not necessary for swap
  for (vector<unsigned int>::const_iterator ite = processes->begin(); ite != processes->end(); ++ite){
    unsigned int process = *ite;
    if (counts[solution.assignment[process]] > 1){
      for (vector<unsigned int>::iterator ite2 = zero.begin(); ite2 != zero.end(); ++ite2){
        setShiftDiffPenalty(process, *ite2, -1);
      }
    }
    else {
      for (vector<unsigned int>::iterator ite2 = nonzero.begin(); ite2 != nonzero.end(); ++ite2){
        if (*ite2 != solution.assignment[process]){
          setShiftDiffPenalty(process, *ite2, 1);
        }
      }
    }
  }
}

// ----- Conflict Constraints -----
void MachineReassignment::Constraints::initConflictConstraints(){
  for (unsigned int s = 0; s < mr->numServices; ++s){
    conflictConstraints.push_back(new ConflictConstraint(this, inf, s));
  }
}
void MachineReassignment::Constraints::prepareConflictConstraints(){
  for (unsigned int s = 0; s < solver->numServices; ++s){
    conflictConstraints[solver->services[s].id]->prepare(solver);
  }
}
